use maud::{Markup, Render, html};

use crate::ui::{UIWidget, color::UIColor};

pub enum BorderSize {
    _0,
    _2,
    _4,
    _8,
}

impl BorderSize {
    pub const fn to_value(&self) -> &str {
        match self {
            BorderSize::_0 => "0",
            BorderSize::_2 => "2",
            BorderSize::_4 => "4",
            BorderSize::_8 => "8",
        }
    }
}

pub enum BorderSide {
    X,
    Y,
    Start,
    End,
    Top,
    Right,
    Bottom,
    Left,
}

impl BorderSide {
    pub const fn to_value(&self) -> &str {
        match self {
            BorderSide::X => "x",
            BorderSide::Y => "y",
            BorderSide::Start => "s",
            BorderSide::End => "e",
            BorderSide::Top => "t",
            BorderSide::Right => "r",
            BorderSide::Bottom => "b",
            BorderSide::Left => "l",
        }
    }
}

pub enum BorderStyle {
    Solid,
    Dashed,
    Dotted,
    Double,
    Hidden,
    None,
}

impl BorderStyle {
    pub const fn to_value(&self) -> &str {
        match self {
            BorderStyle::Solid => "border-solid",
            BorderStyle::Dashed => "border-dashed",
            BorderStyle::Dotted => "border-dotted",
            BorderStyle::Double => "border-double",
            BorderStyle::Hidden => "border-hidden",
            BorderStyle::None => "border-none",
        }
    }
}

#[allow(non_snake_case)]
pub fn Border<T: UIWidget + 'static>(inner: T) -> BorderWidget {
    BorderWidget(Box::new(inner), None, None, None, None)
}

pub struct BorderWidget(
    Box<dyn UIWidget>,
    Option<BorderSize>,
    Option<BorderSide>,
    Option<Box<dyn UIColor>>,
    Option<BorderStyle>,
);

impl BorderWidget {
    #[must_use]
    pub fn size(mut self, size: BorderSize) -> Self {
        self.1 = Some(size);
        self
    }

    #[must_use]
    pub const fn side(mut self, side: BorderSide) -> Self {
        self.2 = Some(side);
        self
    }

    #[must_use]
    pub const fn style(mut self, style: BorderStyle) -> Self {
        self.4 = Some(style);
        self
    }

    #[must_use]
    pub fn color<C: UIColor + 'static>(mut self, color: C) -> Self {
        self.3 = Some(Box::new(color));
        self
    }

    fn border_class(&self) -> String {
        if let Some(side) = &self.2 {
            if let Some(size) = &self.1 {
                return format!("border-{}-{}", side.to_value(), size.to_value());
            }
        } else if let Some(size) = &self.1 {
            return format!("border-{}", size.to_value());
        }

        "border".to_owned()
    }
}

impl Render for BorderWidget {
    fn render(&self) -> Markup {
        self.render_with_class("")
    }
}

impl UIWidget for BorderWidget {
    fn can_inherit(&self) -> bool {
        true
    }

    fn base_class(&self) -> Vec<String> {
        let mut ret = vec![self.border_class()];

        if let Some(color) = &self.3 {
            ret.push(format!("border-{}", color.color_class()));
        }

        if let Some(style) = &self.4 {
            ret.push(style.to_value().to_string());
        }

        ret
    }

    fn extended_class(&self) -> Vec<String> {
        let mut c = self.base_class();
        c.extend_from_slice(&self.0.extended_class());
        c
    }

    fn render_with_class(&self, class: &str) -> Markup {
        if self.0.as_ref().can_inherit() {
            self.0
                .as_ref()
                .render_with_class(&format!("{} {class}", self.base_class().join(" ")))
        } else {
            html! {
                div class=(format!("{} {class}", self.base_class().join(" "))) {
                    (self.0.as_ref())
                }
            }
        }
    }
}
